package tsuteto.spelunker.blockaspect;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.EnumSet;
import java.util.List;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.DamageSource;
import tsuteto.spelunker.damage.SpelunkerDamageSource;

/**
 * For hardcore spelunker for now
 *
 * @author Tsuteto
 */
public class BlockAspectHC<T>
{
    public enum Type
    {
        /** Give damage when left clicking */
        UnclickableLeft,
        /** Give damage when right clicking */
        UnclickableRight,
        /** Give damage when digging */
        Undiggable,
        /** Give damage when destroying */
        Unbreakable,
        /** Give damage in collision */
        Untouchable
    }

    public static final Map<Type, List<BlockAspectHC>> registry = new EnumMap<Type, List<BlockAspectHC>>(Type.class)
    {
        {
            for (Type t : Type.values())
            {
                put(t, new ArrayList<BlockAspectHC>());
            }
        }
    };

    public static void init()
    {
        create(Material.glass, SpelunkerDamageSource.glass, 1.5F)
                .setUnbreakable().register();

        create(Block.cactus.blockID, DamageSource.cactus, 1.0F)
                .setUnclickableLeft().setAllowEfficientTool(EnumSet.of(Type.UnclickableLeft))
                .register();

        create(Block.mycelium.blockID, SpelunkerDamageSource.mycelium, 1.0F)
                .setUnbreakable().register();

        create(Block.furnaceBurning.blockID, DamageSource.onFire, 2.0F)
                .setUnclickableLeft().setUnclickableRight()
                .setAllowEfficientTool(EnumSet.of(Type.UnclickableLeft))
                .setUntouchable().register();

        create(Block.pumpkinLantern.blockID, DamageSource.onFire, 0.5F)
                .setUnclickableLeft().setUntouchable()
                .setAllowEfficientTool(EnumSet.of(Type.UnclickableLeft))
                .register();

        create(Block.slowSand.blockID, SpelunkerDamageSource.soulSand, 1.0F)
                .setUnclickableLeft().setUntouchable().register();

        create(Block.leaves.blockID, SpelunkerDamageSource.grass, 0.5F)
                .setUntouchable().register();
    }

    public static BlockAspectClass create(Class material, DamageSource dmgsrc, float amount)
    {
        return new BlockAspectClass(material, dmgsrc, amount);
    }

    public static BlockAspectMaterial create(Material material, DamageSource dmgsrc, float amount)
    {
        return new BlockAspectMaterial(material, dmgsrc, amount);
    }

    public static BlockAspectId create(int blockId, DamageSource dmgsrc, float amount)
    {
        return new BlockAspectId(blockId, dmgsrc, amount);
    }

    public static boolean matches(EntityPlayer player, Block block, Type type)
    {
        for (BlockAspectHC entry : registry.get(type))
        {
            if (entry.matchesCondition(block, player, type))
            {
                return true;
            }
        }
        return false;
    }

    public static boolean applyDamage(EntityPlayer player, Block block, Type type)
    {
        for (BlockAspectHC entry : registry.get(type))
        {
            if (entry.matchesCondition(block, player, type))
            {
                entry.applyDamage(player);
                return true;
            }
        }
        return false;
    }

    public boolean isUnclickableLeft = false;
    public boolean isUnclickableRight = false;
    public boolean isUndiggable = false;
    public boolean isUnbreakable = false;
    public boolean isUntouchable = false;

    protected T condition;
    protected EnumSet<Type> allowEfficientTool = EnumSet.noneOf(Type.class);

    protected DamageSource dmgsrc;
    protected float dmgAmount;

    public BlockAspectHC(T cond, DamageSource dmgsrc, float amount)
    {
        this.condition = cond;
        this.dmgsrc = dmgsrc;
        this.dmgAmount = amount;
    }

    public boolean matchesCondition(Block block, EntityPlayer player, Type type)
    {
        ItemStack equipped = player.getCurrentEquippedItem();
        boolean isBreakable = equipped != null ? equipped.getStrVsBlock(block) > 1.0 : false;

        if (allowEfficientTool.contains(type) && isBreakable) return false;
        return matches(block);
    }

    protected boolean matches(Block block)
    {
        return condition.equals(block);
    }

    public void applyDamage(EntityPlayer player)
    {
        player.attackEntityFrom(dmgsrc, dmgAmount);
    }

    public BlockAspectHC<T> setUnclickableLeft()
    {
        isUnclickableLeft = true;
        return this;
    }

    public BlockAspectHC<T> setUnclickableRight()
    {
        isUnclickableRight = true;
        return this;
    }

    public BlockAspectHC<T> setUndiggable()
    {
        isUndiggable = true;
        return this;
    }

    public BlockAspectHC<T> setUnbreakable()
    {
        isUnbreakable = true;
        return this;
    }

    public BlockAspectHC<T> setUntouchable()
    {
        isUntouchable = true;
        return this;
    }

    public BlockAspectHC<T> setAllowEfficientTool(EnumSet enumset)
    {
        allowEfficientTool = enumset;
        return this;
    }

    public void register()
    {
        if (isUnclickableLeft)
            registry.get(Type.UnclickableLeft).add(this);
        if (isUnclickableRight)
            registry.get(Type.UnclickableRight).add(this);
        if (isUndiggable)
            registry.get(Type.Undiggable).add(this);
        if (isUnbreakable)
            registry.get(Type.Unbreakable).add(this);
        if (isUntouchable)
            registry.get(Type.Untouchable).add(this);
    }
}
